from argparse import ArgumentParser
from tqdm import tqdm
import json
import os
import pandas
from typing import List

def get_epsilon(path: str):
    path = path.split('/')[-1]
    epsilon = path.split('-')[1]
    return float(epsilon)

def get_strategy(path: str):
    strategies = ['greedy_v2', 'hard_greedy', 'anti_greedy', 'greedy', 'random']
    for strategy in strategies:
        if strategy in path:
            return strategy

def get_space(path: str):
    spaces = ['input', 'projected']
    if 'input' in path:
        return 'input'

    projected_dims = [3000, 2048, 1024]
    if 'projected' in path:
        for p in projected_dims:
            if str(p) in path:
                return 'projected' + str(p)

def get_evals(path: str):
    '''
    Returns (adv_eval, nat_eval) from the json stored as path
    '''
    info = json.load(open(path, 'r'))
    info = dict(info)
    return info['adv_prec1']/100.0, info['nat_prec1']/100.0

def get_row(path: str):

    epsilon = float(path.split('-')[-2])
    result = {'eps': epsilon}
    result['adv_eval'], result['nat_eval'] = get_evals(path)
    result['strategy'] = 'feature_l2'
    result['budget'] = int(path.split('/')[-4])
    # if result['strategy'] not in ['random', None]:
        # result['strategy'] += '-' + get_space(path)
    # if result['strategy'] != 'random':
        # result['space'] = get_space(path)
    return result


def get_dataframe(paths: List[str]):
    df = pandas.DataFrame(columns=['strategy', 'budget', 'eps', 'adv_eval', 'nat_eval'])
    ll = []
    for path in tqdm(paths):
        df = df.append(get_row(path), ignore_index=True)
    return df

# def main():
    # parser = ArgumentParser()
    # parser.add_argument('--json_paths', nargs='+', required=True)
    # parser.add_argument('--save_path', type=str, required=True)
    # 
    # args = parser.parse_args()
# 
    # dataframe = get_dataframe(args.json_paths)
    # print(set(dataframe['strategy']))
    # print(dataframe.to_string())
    # dataframe.to_pickle(args.save_path)

def get_row_arc(path):
    epsilon = float(os.path.basename(path).split('-')[1])
    result = {'eps': epsilon}
    result['adv_eval'], result['nat_eval'] = get_evals(path)
    print(result)
    # result['strategy'] = get_strategy(path)
    result['strategy'] = 'random'
    result['budget'] = int(path.split('/')[-4])
    # if result['strategy'] not in ['random', None]:
    # result['strategy'] += '-' + get_space(path)
    # if result['strategy'] != 'random':
    # result['space'] = get_space(path)
    return result



def get_dataframe(paths: List[str]):
    df = pandas.DataFrame(columns=['strategy', 'budget', 'eps', 'adv_eval', 'nat_eval'])
    ll = []
    for path in tqdm(paths):
        row = get_row_arc(path)
        if row['eps'] not in [0.001, 0.002, 0.004]:
            df = df.append(row, ignore_index=True)
    return df

def main():
    parser = ArgumentParser()
    parser.add_argument('--json_paths', nargs='+', required=True)
    parser.add_argument('--save_path', type=str, required=True)

    args = parser.parse_args()

    dataframe = get_dataframe(args.json_paths)
    # print(set(dataframe['strategy']))
    print(dataframe.to_string())
    dataframe.to_pickle(args.save_path)

main()

