import robustness
from robustness import model_utils, datasets, \
    defaults, imagenet_models, cifar_models,  \
    train, data_augmentation as da
from torchvision.datasets import CIFAR10
import torch
from poisoning_adversary import *
import pickle

class PoisonedDataSet(robustness.datasets.DataSet):
    def __init__(self, **kwargs):
        super(PoisonedDataSet, self).__init__(**kwargs)
    def save_mask(self):
        raise NotImplementedError

class CIFAR(PoisonedDataSet):

    def __init__(self, data_path, **kwargs):
        """
        data_path
        """
        ds_kwargs = {
            'num_classes': 10,
            'mean': torch.tensor([0.4914, 0.4822, 0.4465]),
            'std': torch.tensor([0.2023, 0.1994, 0.2010]),
            'custom_class': None,
            'label_mapping': None,
            'transform_train': [],
            'transform_test': []
        }
        ds_kwargs = self.override_args(ds_kwargs, kwargs)
        super(CIFAR, self).__init__(ds_name='cifar', \
            data_path=data_path, **ds_kwargs)
        self.train_set = CIFAR10(
            root=data_path, 
            train=True,
        )
        self.test_set = CIFAR10(
            root=data_path,
            train=False,
            transform=da.TEST_TRANSFORMS_DEFAULT(32)
        )
        self.data_path = data_path

    def make_loaders(self,
        batch_size: int,
        shuffle: bool,
        selection_func,
        ) -> torch.utils.data.DataLoader:
        '''
        selection_func: applied to the input points,
        classwise from the CIFAR training set, to select
        points whose labels are flipped.
        kwargs: arguments that get passed on to the
        robustness.datasets.CIFAR make_loaders method, 
        such as batch_size and shuffle.
        '''
        test_loader = torch.utils.data.DataLoader(self.test_set, 
            batch_size=batch_size, shuffle=shuffle)
        train_dataset = get_poisoned_dataset(
            dataset=self.train_set,
            selection_func=selection_func,
            transform=da.TRAIN_TRANSFORMS_DEFAULT(32),
            return_dataset=True
        )
        train_loader = torch.utils.data.DataLoader(
            train_dataset,
            batch_size=batch_size,
            shuffle=shuffle,
        )
        return train_loader, test_loader
    
    def save_mask(self, mask_path: str, selection_func):
        train_flip_mask = get_poisoned_dataset(
            dataset=self.train_set,
            selection_func=selection_func,
            transform=da.TRAIN_TRANSFORMS_DEFAULT(32),
            return_dataset=False
        )
        with open(mask_path, 'wb') as f:
            pickle.dump(train_flip_mask, f)

    def get_model(self, arch, pretrained):
        if pretrained:
            raise ValueError('CIFAR does not support pytorch_pretrained=True')
        return cifar_models.__dict__[arch](num_classes=self.num_classes)


DATASETS = {
    'cifar': CIFAR
}
